package cn.com.qmth.stmms.ms.log.aop;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.*;

/**
 * @Description: 纪检审查aop
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/19
 */
@Aspect
@Component
public class MarkLogAop {

    private static Logger LOGGER = LoggerFactory.getLogger(MarkLogAop.class);

    private final String TOKEN = "token",
            MARK_USER = "markUser",//登录用户
            COLLECTOR = "collector",//采集用户
            PARAM_ARGS = "args",//方法参数
    //            PARAM_ARG_NAMES = "argNames",//方法参数名
    ACTION = "action",//动作
            SAMPLING = "sampling",//标准卷设置
            LEVELING = "leveling",//定档
            REJECT = "reject",//档位打回
            PAPER = "paper",//paper
            STAGE = "stage",//stage
            LEVEL = "level",//level
            BODY = "body",//body
            RESULT = "result",//result
            DEFAULT_RESULT = "-";//默认值

    @Autowired
    private MarkLogRepo markLogRepo;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    SqlUtil sqlUtil;

    /**
     * 标准卷、定档、档位打回切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.PaperApi.marking(..))")
    public void paperAspect() {
    }

    /**
     * 分档、打分、回评档位、回评分数、档位打回回评切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.MarkTaskApi.marking(..))")
    public void markAspect() {
    }

//    /**
//     * 采集端采集试卷切入点
//     */
//    @Pointcut("execution(* cn.com.qmth.stmms.ms.collect.api.CollectApi.imageUploadSheet(..))")
//    public void collectAspect() {
//    }

    /**
     * 标准卷设置、定档设置、档位打回切入
     *
     * @param joinPoint
     * @param result
     * @throws Exception
     */
    @AfterReturning(value = "paperAspect()", returning = RESULT)
    public void afterReturnPaperPoint(JoinPoint joinPoint, Object result) throws Exception {
        Map<String, Object> map = getAspectCommon(joinPoint, 0);
        JSONArray jsonArgsArray = (JSONArray) map.get(PARAM_ARGS);
        MarkUser markUser = (MarkUser) map.get(MARK_USER);
        Paper paper = JSONObject.parseObject(((JSONObject) jsonArgsArray.get(0)).toJSONString(), Paper.class);
        JSONObject jsonObjectResult = (JSONObject) jsonArgsArray.get(1);
        String action = null;
        Integer operType = null;
        if (Objects.nonNull(jsonObjectResult) && Objects.nonNull(jsonObjectResult.get(ACTION))) {
            action = (String) jsonObjectResult.get(ACTION);
            if (Objects.equals(SAMPLING, action)) {
                //标准卷设置
                operType = MarkLogOperType.CRITERION_PAPER_SET.getId();
            } else if (Objects.equals(LEVELING, action)) {
                //定档设置
                operType = MarkLogOperType.ONE_CLICK_LEVEl.getId();
            } else if (Objects.equals(REJECT, action)) {
                //档位打回设置
                operType = MarkLogOperType.CALLBACK_LEVEl.getId();
            }
        }
        //标准卷设置、定档设置、档位打回start
        MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), operType, markUser.getSubject().toString(), paper.getExamNumber());
        String operResult = DEFAULT_RESULT;
        if (Objects.nonNull(markLogPrev)) {
            operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
        }
        MarkLog markLog = new MarkLog(markUser.getId(), markUser.getName(), markUser.getRole(), markUser.getSubject(), paper.getExamNumber(), paper.getStudentName(), operType, paper.getWorkId(), paper.getId(), MarkStage.LEVEL, operResult, String.valueOf(jsonObjectResult.get(LEVEL)), null);
        LOGGER.info("markLog:{}", JSONObject.toJSONString(markLog));
        //标准卷设置、定档设置、档位打回end
        markLogRepo.save(markLog);
    }

    /**
     * 分档、打分、回评档位、回评分数、档位打回回评切入
     *
     * @param joinPoint
     * @param result
     * @throws Exception
     */
    @AfterReturning(value = "markAspect()", returning = RESULT)
    public void afterReturnMarkPoint(JoinPoint joinPoint, Object result) throws Exception {
        Map<String, Object> map = getAspectCommon(joinPoint, 0);
        JSONArray jsonArgsArray = (JSONArray) map.get(PARAM_ARGS);
        MarkUser markUser = (MarkUser) map.get(MARK_USER);
        JSONObject aopResult = JSONObject.parseObject(JSONObject.toJSONString(result));
        boolean isRejected = false, oldRejected = false;
        if (Objects.nonNull(aopResult)) {
            MarkTaskDTO resultBody = JSONObject.parseObject(((JSONObject) aopResult.get(BODY)).toJSONString(), MarkTaskDTO.class);
            isRejected = resultBody.isRejected();//新的是否撤档
            oldRejected = resultBody.isOldRejected();//旧的是否撤档
        }
        JSONObject jsonObject = (JSONObject) jsonArgsArray.get(0);
        Paper paper = JSONObject.parseObject(((JSONObject) jsonObject.get(PAPER)).toJSONString(), Paper.class);
        JSONObject jsonObjectResult = (JSONObject) jsonArgsArray.get(1);
        LOGGER.info("jsonObject:{},paper:{},jsonObjectResult:{}", jsonObject, paper, jsonObjectResult);
        Integer operType = MarkLogOperType.LEVEL.getId();//分档,第一次为分档，之后为回评档位,档位打回后为档位打回回评
        //分档、打分、回评档位、回评分数、档位打回回评start
        MarkStage markStage = MarkStage.valueOf(String.valueOf(jsonObjectResult.get(STAGE)));
        if (markStage.ordinal() == 1 && paper.isMarkByLeader()) {//分档,如果已经有科组长定档，则不允许评卷员再分档
            return;
        } else if (markStage.ordinal() == 1 && oldRejected && !isRejected) {//档位打回回评
            operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();
        } else if (markStage.ordinal() == 2) {//打分
            operType = MarkLogOperType.SCORE.getId();
        }
        //先查是否有档位或分数打回
        String operResult = DEFAULT_RESULT;
        if (markStage.ordinal() == 1) {//分档
            //先查是否有档位打回回评的数据
            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), MarkLogOperType.CALLBACK_ONCE_LEVEL.getId(), markUser.getSubject().toString(), paper.getExamNumber());
            if (Objects.isNull(markLogPrev)) {//如果为null，则再查分档和回评档位
                if (oldRejected && !isRejected) {//档位打回
                    operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();
                } else {
                    operType = selectOperType(markUser.getId(), MarkLogOperType.LEVEL.getId(), MarkLogOperType.ONCE_LEVEl.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                    markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), operType, markUser.getSubject().toString(), paper.getExamNumber());
                    if (Objects.nonNull(markLogPrev)) {
                        operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                    } else {
                        if (operType != MarkLogOperType.LEVEL.getId()) {
                            markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), MarkLogOperType.LEVEL.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                            operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                        }
                    }
                }
            } else {
                operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();//档位打回
                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
            }
        } else if (markStage.ordinal() == 2) {//打分
            operType = selectOperType(markUser.getId(), MarkLogOperType.SCORE.getId(), MarkLogOperType.ONCE_SCORE.getId(), markUser.getSubject().toString(), paper.getExamNumber());
            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), operType, markUser.getSubject().toString(), paper.getExamNumber());
            if (Objects.nonNull(markLogPrev)) {
                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
            } else {
                if (operType != MarkLogOperType.SCORE.getId()) {
                    markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), MarkLogOperType.SCORE.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                    operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                }
            }
        }
        MarkLog markLog = new MarkLog(markUser.getId(), markUser.getName(), markUser.getRole(), markUser.getSubject(), paper.getExamNumber(), paper.getStudentName(), operType, paper.getWorkId(), paper.getId(), markStage, operResult, String.valueOf(jsonObjectResult.get(RESULT)), null);
        LOGGER.info("markLog:{}", JSONObject.toJSONString(markLog));
        //分档、打分、回评档位、回评分数、档位打回、档位打回回评end
        markLogRepo.save(markLog);
    }

//    /**
//     * 采集切入
//     *
//     * @param joinPoint
//     * @param result
//     * @throws Exception
//     */
//    @AfterReturning(value = "collectAspect()", returning = RESULT)
//    public void afterReturnCollectPoint(JoinPoint joinPoint, Object result) throws Exception {
//        Map<String, Object> map = getAspectCommon(joinPoint, 1);
//        JSONArray jsonArgsArray = (JSONArray) map.get(PARAM_ARGS);
//        MarkUser markUser = (MarkUser) map.get(MARK_USER);
////        JSONObject aopResult = JSONObject.parseObject(JSONObject.toJSONString(result));
//
//        // 参数 workdId, subjectId, examNumber
//        JSONObject arg0 = (JSONObject) jsonArgsArray.get(0);
//        Long workId = JSONObject.parseObject(arg0.toJSONString(), Long.class);
//        JSONObject arg1 = (JSONObject) jsonArgsArray.get(1);
//        int subjectId = JSONObject.parseObject(arg1.toJSONString(), int.class);
//        Subject subject = Subject.values()[subjectId - 1];
//        JSONObject arg3 = (JSONObject) jsonArgsArray.get(2);
//        String examNumber = JSONObject.parseObject(arg3.toJSONString(), String.class);
//
//        //学生信息
//        Student student = studentRepo.findByWorkIdAndExamNumber(workId, examNumber);
//
//        //试卷信息
//        List<Paper> paperList = paperRepo.findByWorkIdAndExamNumber(workId, examNumber);
//        Paper paper = paperList.get(0);
//
//        Integer operType = MarkLogOperType.COLLECT.getId();
//
//        MarkLog markLog = new MarkLog();
//        markLog.setCreateUserId(markUser.getId());
//        markLog.setCreateUserName(markUser.getName());
//        markLog.setCreateRole(markUser.getRole().getName());
//        markLog.setSubject(subject.toString());
//        markLog.setExamNumber(examNumber);
//        markLog.setStudentName(student.getName());
//        markLog.setOperType(operType);
//        markLog.setWorkId(workId);
//        markLog.setPaperId(paper.getId());
//        markLog.setStage(MarkStage.INIT);
//        //识别方式
//        markLog.setRemark("");
//        //操作时间
//        markLog.setCreateTime(new Date());
//        //todo
//        //登录时间
//        //退出时间
//        LOGGER.info("markLog:{}", JSONObject.toJSONString(markLog));
//        //分档、打分、回评档位、回评分数、档位打回、档位打回回评end
//        markLogRepo.save(markLog);
//    }

    /**
     * 查询处于哪个档位公用
     *
     * @param createUserId
     * @param operTypeStart
     * @param operTypeEnd
     * @param subject
     * @param examNumber
     * @return
     */
    public Integer selectOperType(Long createUserId, Integer operTypeStart, Integer operTypeEnd, String subject, String examNumber) {
        String sql = "select t.oper_type as operType, count(*) as result from mark_log as t where t.create_user_id = ? and (t.oper_type = ? or t.oper_type = ?) and t.subject = ? and t.exam_number = ? group by t.oper_type ";
        List list = sqlUtil.execSqlForMap(sql, createUserId, operTypeStart, operTypeEnd, subject, examNumber);
        Integer operType = operTypeStart;
        if (Objects.nonNull(list) && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                Map m = (Map) list.get(i);
                Integer type = Integer.parseInt(String.valueOf(m.get("operType")));
                if (type.intValue() == operTypeStart.intValue()) {
                    operType = operTypeEnd;
                    break;
                } else {
                    operType = type;
                    break;
                }
            }
        }
        return operType;
    }

    /**
     * 切面公用方法
     *
     * @param joinPoint
     * @param type
     * @return
     * @throws Exception
     */
    public Map getAspectCommon(JoinPoint joinPoint, int type) throws Exception {
        String className = joinPoint.getSignature().getDeclaringTypeName();
        String methodName = joinPoint.getSignature().getName();
        LOGGER.info("{} aspect path:{}", this.getClass().getMethods()[0], className + "." + methodName);
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        HttpSession session = request.getSession();
        String token = (String) session.getAttribute(TOKEN);
        if (Objects.isNull(token)) {
            throw new Exception("请先登录");
        }
        MarkUser markUser = null;
        switch (type) {
            case 0:
                markUser = (MarkUser) session.getAttribute(MARK_USER);
                break;
            default:
                markUser = (MarkUser) session.getAttribute(COLLECTOR);
                break;
        }
        LOGGER.info("markUser:{}", JSONObject.toJSONString(markUser));
        Object[] args = joinPoint.getArgs(); // 参数值
//        String[] argNames = ((MethodSignature) joinPoint.getSignature()).getParameterNames();//参数名
        JSONArray jsonArgsArray = JSONObject.parseArray(JSONObject.toJSONString(args));
//        JSONArray jsonArgNamesArray = JSONObject.parseArray(JSONObject.toJSONString(argNames));
        LOGGER.info("jsonArgsArray:{},jsonArgNamesArray:{}", jsonArgsArray);
//        LOGGER.info("jsonArgNamesArray:{}", jsonArgNamesArray);
        Map<String, Object> map = new HashMap<>();
        map.put(PARAM_ARGS, jsonArgsArray);
//        map.put(PARAM_ARG_NAMES, jsonArgNamesArray);
        map.put(MARK_USER, markUser);
        return map;
    }
}
